package de.beyondjava.SimpleClassParser;

import static de.beyondjava.SimpleClassParser.RegularExpressions.*

import java.util.regex.Matcher

import com.google.gson.Gson

/**
 * Description of a class, including a list of its variables and methods (but ignoring the parameter lists of the methods)
 */
public class ClassDefinition {
	String packageName;
	String className;
	List<String> variables = []
	List<String> methods = []
	List<String> modifiers=[];
	List<String> annotations=[];

	/**
	 * extract the variables and methods from the implementation
	 */
	ClassDefinition (String p_packageName, String p_classDeclaration, String p_impl) {
		Matcher m = p_classDeclaration =~ ANNOTATION
		m.each({annotations = annotations + it[0].trim()})
		// simplify expression by removing annotations
		p_classDeclaration = p_classDeclaration.replaceAll(ANNOTATION,"")

		List<String> words = p_classDeclaration.split("class");
		words = words.findAll{it?.trim()}
		className = words[-1].trim()
		if (words.size()>1)
		{
			modifiers = words[0..-2].collect{it?.trim()}
		}

		packageName=p_packageName
		def implementation=removeImplementationDetails(p_impl)
		extractVariablesAndMethods(implementation)
	}

	/**
	 * In order to keep things simple, we remove the implementation of the methods.
	 */
	public String removeImplementationDetails(String p_theSourcecode) {
		// This regular expression removes everything enclosed between curly braces
		// we have to apply it repeatedly because regular expressions are badly suited for counting braces
		String s = p_theSourcecode
		while (s.contains('{') && s.contains('}'))
		{
			s = s.replaceAll(JAVA_BLOCK_REGEXP,"\n")
		}
		return s
	}

	private void extractVariablesAndMethods(String p_impl) {
		try
		{
			Matcher m = p_impl =~ VARIABLE_OR_METHOD_REGEXP
			def meths = m.findAll{it[0].endsWith(")")}.collect{it[0].trim()}
			methods = meths.collect{ new MethodDefinition(it)}
			def vars = m.findAll{!(it[0].endsWith(")"))}.collect{it[0].trim()}
			variables = vars.collect{ new VariableDefinition(it)}
		}
		catch (Exception fehler)
		{
			println "Exception $fehler"
			println p_impl
		}
	}

	public String toString()
	{
		Gson g = new Gson();
		g.toJson(this)
	}

	public ClassDefinition clone()
	{
		Gson g = new Gson();
		g.fromJson(g.toJson(this), ClassDefinition.class)
	}
}
