package de.beyondjava.SimpleClassParser

import java.util.regex.Matcher
import java.util.regex.Pattern
import static RegularExpressions.*

/**
 * This is a simple class parser that reads every java or groovy class in a folder
 * and returns a list of class descriptions.
 */
class SimpleClassParser {

	/** Parse a file or a folder recursively. */
	public List<ClassDefinition> parse(String p_folderOrFilename) {
		File f = new File(p_folderOrFilename)
		parse(f);
	}

	/** Parse a file or a folder recursively. */
	public List<ClassDefinition> parse(File p_fileOrFolder) {
		List<ClassDefinition> classDefinitions = []
		if (p_fileOrFolder.exists()) {
			if (p_fileOrFolder.isDirectory()) {
				List<File> files = p_fileOrFolder.listFiles();

				List<File> folders = files.findAll({File g -> g.isDirectory()})
				folders.each {parse it}
				List<File> classFiles = files.findAll({File g -> g.name.endsWith(".java" ) || g.name.endsWith(".groovy")})
				classFiles.each{classDefinitions = classDefinitions + parseASingleFile(it)}
			}
		}
		else {
			println "File or folder '$p_fileOrFolder' not found"
		}
		return classDefinitions
	}

	private List<ClassDefinition> parseASingleFile(File p_file) {
		String theFile = p_file.text
		theFile = removeStrings(theFile)
		theFile = removeComments(theFile)
		String currentPackage=findPackage(theFile);
		if(!currentPackage) return [];

		return splitIntoClasses(theFile, currentPackage)
	}


	private String findPackage(String theFile) {
		Matcher m = theFile =~ packageRegExp
		if (m.count != 1) {
			println "This is neither a java nor a groovy file"
			return null
		}
		// the package name is the third expression enclosed by parantheses
		return m[0][3]
	}

	private List<ClassDefinition> splitIntoClasses(String theFile, String p_currentPackage) {
		List<ClassDefinition> classes = []
		Matcher m = theFile =~ CLASS_REGEXP
		def implementations = theFile.split(CLASS_REGEXP.toString())
		// remove the text before the first class definition
		implementations = implementations[1..-1]
		m.eachWithIndex{ List<String> classMatch, int index -> classes = classes + extractClass(classMatch, implementations[index], p_currentPackage)}

		return classes;
	}
	private ClassDefinition extractClass(List<String> p_classPatternMatch, String p_implementation, String p_currentPackage)
	{
		// ignore the curly braces surrounding the implementation
		String classBody = p_implementation?.trim()[1..-2]
		String classDeclaration = p_classPatternMatch[0]

		new ClassDefinition(p_currentPackage, classDeclaration, classBody)
	}

	/** Keep things simple by removing string literals. */
	private String removeStrings(String p_theFile)
	{
		p_theFile.replaceAll(STRING_REGEXP, "null")
				.replaceAll(STRING2_REGEXP, "null")
				.replaceAll(STRING3_REGEXP, "null")
				.replaceAll(STRING4_REGEXP, "null")
	}

	/** Keep things simple by removing comments (both single line and multi line comments). */
	private String removeComments(String p_theFile)
	{
		p_theFile.replaceAll(COMMENT_REGEXP,"")
	}
}
