package merge_jsf_jsr303

import de.beyondjava.SimpleClassParser.ClassDefinition
import de.beyondjava.SimpleClassParser.SimpleBeanParser
import de.beyondjava.SimpleClassParser.VariableDefinition

class Merger {
	/**
	 * Analyzes every input text of every JSF file in a folder.
	 * If the bean variable of an input field bears a bean validation annotation,
	 * the JSF file is modified to contain the same information.
	 * @param p_sourceFolder
	 * @param p_jsfFolder
	 * @param p_targetJSFFolder
	 */
	void mergeBeanValidationIntoJSF(String p_sourceFolder, String p_jsfFolder, String p_targetJSFFolder)
	{
		SimpleBeanParser parser = new SimpleBeanParser().parse(p_sourceFolder)
		List<ClassDefinition> beans = parser.beans
		List<VariableDefinition> vars = parser.vars
		enrichEachJSFFile(p_jsfFolder, p_targetJSFFolder, vars)
	}

	void enrichEachJSFFile(String p_jsfFolder, String p_targetJSFFolder, List<VariableDefinition> p_vars)
	{
		File g;
		List<File> files = []
		new File(p_jsfFolder).eachFileRecurse({if (it.isFile() && it.name.endsWith(".xhtml")) files = files + it })
		List<File> targetFiles = files.collect({File f -> new File(f.absolutePath.replace(p_jsfFolder, p_targetJSFFolder))})
		files.eachWithIndex({File source, int index -> enrichJSFFile(source, targetFiles[index], p_vars) })
	}

	void enrichJSFFile(File p_sourceFile, File p_targetFile, List<VariableDefinition> vars)
	{
		def jsf = new XmlParser().parse(p_sourceFile);
		def inputs = jsf['**']
				inputs.each{ println it.name()}
		println "-------------------------------------------------"
		def inputFields = inputs.findAll{ it.name().toString().endsWith("inputText")}
		inputFields.each{ addTags(it, vars)}
//		inputFields.each{ println it['@value']}

//		inputs.each{ println it.name()}
//		println inputs
//		println "$p_sourceFile --> $p_targetFile"
//		println "$jsf"
		def writer = new StringWriter()
		new XmlNodePrinter(new PrintWriter(writer)).print(jsf)
		def result = writer.toString()
		println result
	}

	void addTags(def p_node, List<VariableDefinition> vars)
	{
		String value = p_node.@value
		if (value[0..1]=="#{" && value[-1]=="}")
		{
			String potentialVariable = value[2..-2]
			VariableDefinition match = vars.find{VariableDefinition v -> v.name == potentialVariable}
			if (match)
			{
				if (match.annotations.any{it=="@NotNull"})
				{
					p_node.@required="true"
				}
				def sizeTag = match.annotations.find{it[0..5]=="@Size("}
				if (sizeTag)
				{
					if (sizeTag[-1]==")")
					{
						String[] sizes = sizeTag[6..-2].split(",")
						sizes.each{String s -> if (s.trim().startsWith("max")) addMaxLengthTag(p_node, s) }
						sizes.each{String s -> if (s.trim().startsWith("min")) p_node.@required="true" }

					}
					else
					{
						println "Illegal syntax at $value: $sizeTag"
					}

				}
			}
		}
	}

	void addMaxLengthTag(def p_node, String p_maxLength)
	{
		String[] terms = p_maxLength.split("=")
		String maxLength = terms[1].trim()
		int iMaxLength = new Integer(maxLength).intValue()
		p_node.@maxlength="$iMaxLength"
	}
}
